@@ -1,7 +1,7 @@
 
-#   $Id: Oracle.pm,v 1.55 1998/08/03 19:43:39 timbo Exp $
+#   $Id: Oracle.pm,v 1.61 1998/12/16 00:23:12 timbo Exp $
 #
-#   Copyright (c) 1994,1995,1996,1997 Tim Bunce
+#   Copyright (c) 1994,1995,1996,1997,1998 Tim Bunce
 #
 #   You may distribute under the terms of either the GNU General Public
 #   License or the Artistic License, as specified in the Perl README file,
@@ -10,7 +10,7 @@
 
 require 5.002;
 
-$DBD::Oracle::VERSION = '0.53';
+$DBD::Oracle::VERSION = '0.55';
 
 my $ORACLE_ENV  = ($^O eq 'VMS') ? 'ORA_ROOT' : 'ORACLE_HOME';
 
@@ -21,10 +21,19 @@ my $ORACLE_ENV  = ($^O eq 'VMS') ? 'ORA_ROOT' : 'ORACLE_HOME';
     use DynaLoader ();
     use Exporter ();
     @ISA = qw(DynaLoader Exporter);
+    %EXPORT_TAGS = (
+	ora_types => [ qw(
+	    ORA_VARCHAR2 ORA_NUMBER ORA_LONG ORA_ROWID ORA_DATE
+	    ORA_RAW ORA_LONGRAW ORA_CHAR ORA_MLSLABEL ORA_NTY
+	    ORA_CLOB ORA_BLOB
+	) ],
+    );
+    Exporter::export_ok_tags('ora_types');
 
-    my $Revision = substr(q$Revision: 1.55 $, 10);
 
-    require_version DBI 0.92;
+    my $Revision = substr(q$Revision: 1.61 $, 10);
+
+    require_version DBI 1.02;
 
     bootstrap DBD::Oracle $VERSION;
 
@@ -99,10 +108,6 @@ my $ORACLE_ENV  = ($^O eq 'VMS') ? 'ORA_ROOT' : 'ORACLE_HOME';
 		warn "Found $name. ".($dbnames{$name} ? "(oratab entry overridden)" : "")."\n"
 		    if $debug;
 		$dbnames{$name} = 0; # exists but false (to distinguish from oratab)
-		if ($name =~ /^([-\w]+)\.([-\w\.]+)/ && !exists $dbnames{$1}) {
-		    warn " also $1. \n" if $debug;
-		    $dbnames{$1} = 0; # exists but false (to distinguish from oratab)
-		}
 	    }
 	    close FH;
 	    last;
@@ -132,9 +137,15 @@ my $ORACLE_ENV  = ($^O eq 'VMS') ? 'ORA_ROOT' : 'ORACLE_HOME';
 
 
     sub connect {
-	my($drh, $dbname, $user, $auth)= @_;
+	my ($drh, $dbname, $user, $auth)= @_;
 
-	if ($dbname){	# application is asking for specific database
+	# If the application is asking for specific database
+	# then we have to mung the
+
+	if (DBD::Oracle::ORA_OCI() >= 8) {
+	    $dbname = $1 if !$dbname && $user =~ s/\@(.*)//;
+	}
+	elsif ($dbname) {
 
 	    # We can use the 'user/passwd@machine' form of user.
 	    # $TWO_TASK and $ORACLE_SID will be ignored in that case.
@@ -161,7 +172,7 @@ my $ORACLE_ENV  = ($^O eq 'VMS') ? 'ORA_ROOT' : 'ORACLE_HOME';
 		if (exists $dbnames{$dbname}) {	# known name
 		    my $dbhome = $dbnames{$dbname};	# local=>ENV, remote=>0
 		    if ($dbhome) {
-			warn "Changing $ORACLE_ENV for $dbname to $dbhome"
+			warn "Changing $ORACLE_ENV for $dbname to $dbhome (to match oratab entry)"
 			    if ($ENV{$ORACLE_ENV} and $dbhome ne $ENV{$ORACLE_ENV});
 			$ENV{ORACLE_SID}  = $dbname;
 			$ENV{$ORACLE_ENV} = $dbhome;
@@ -186,7 +197,7 @@ my $ORACLE_ENV  = ($^O eq 'VMS') ? 'ORA_ROOT' : 'ORACLE_HOME';
 	    'USER' => $user, 'CURRENT_USER' => $user,
 	    });
 
-	# Call Oracle OCI orlon func in Oracle.xs file
+	# Call Oracle OCI logon func in Oracle.xs file
 	# and populate internal handle data.
 	DBD::Oracle::db::_login($dbh, $dbname, $user, $auth)
 	    or return undef;
@@ -209,8 +220,7 @@ my $ORACLE_ENV  = ($^O eq 'VMS') ? 'ORA_ROOT' : 'ORACLE_HOME';
 	    'Statement' => $statement,
 	    });
 
-	# Call Oracle OCI oparse func in Oracle.xs file.
-	# (This will actually also call oopen for you.)
+	# Call Oracle OCI parse func in Oracle.xs file.
 	# and populate internal handle data.
 
 	DBD::Oracle::st::_prepare($sth, $statement, @attribs)
@@ -222,29 +232,86 @@ my $ORACLE_ENV  = ($^O eq 'VMS') ? 'ORA_ROOT' : 'ORACLE_HOME';
 
     sub ping {
 	my($dbh) = @_;
-	# we know that DBD::Oracle prepare does a describe so this will
+	# we know that Oracle 7 prepare does a describe so this will
 	# actually talk to the server and is this a valid and cheap test.
-	return 1 if $dbh->prepare("select SYSDATE from DUAL");
+	my $sth =  $dbh->prepare("select SYSDATE from DUAL");
+	# But Oracle 8 doesn't talk to server unless we describe the query
+	return 1 if $sth && $sth->{NUM_OF_FIELDS};
 	return 0;
     }
 
 
-    sub tables {
+    sub table_info {
 	my($dbh) = @_;		# XXX add qualification
+	# XXX add knowledge of public synonmys views etc
+	# The SYS/SYSTEM should probably be a decode that
+	# prepends 'SYSTEM ' to TABLE_TYPE.
 	my $sth = $dbh->prepare("select
-		NULL		TABLE_CAT,
-		at.OWNER	TABLE_SCHEM,
+		NULL		TABLE_QUALIFIER,
+		at.OWNER	TABLE_OWNER,
 		at.TABLE_NAME,
 		tc.TABLE_TYPE,
-		tc.COMMENTS	TABLE_REMARKS
+		tc.COMMENTS	REMARKS
 	    from ALL_TABLES at, ALL_TAB_COMMENTS tc
 	    where at.OWNER = tc.OWNER
 	    and at.TABLE_NAME = tc.TABLE_NAME
-	");
+	    and at.OWNER <> 'SYS' and at.OWNER <> 'SYSTEM'
+	    order by tc.TABLE_TYPE, at.OWNER, at.TABLE_NAME
+	") or return undef;
 	$sth->execute or return undef;
 	$sth;
     }
 
+    sub type_info_all {
+	my ($dbh) = @_;
+	my $names = {
+          TYPE_NAME		=> 0,
+          DATA_TYPE		=> 1,
+          COLUMN_SIZE		=> 2,
+          LITERAL_PREFIX	=> 3,
+          LITERAL_SUFFIX	=> 4,
+          CREATE_PARAMS		=> 5,
+          NULLABLE		=> 6,
+          CASE_SENSITIVE	=> 7,
+          SEARCHABLE		=> 8,
+          UNSIGNED_ATTRIBUTE	=> 9,
+          FIXED_PREC_SCALE	=>10,
+          AUTO_UNIQUE_VALUE	=>11,
+          LOCAL_TYPE_NAME	=>12,
+          MINIMUM_SCALE		=>13,
+          MAXIMUM_SCALE		=>14,
+        };
+	# Based on the values from Oracle 8.0.4 ODBC driver
+	my $ti = [
+	  $names,
+          [ 'LONG RAW', -4, '2147483647', '\'', '\'', undef, 1, '0', '0',
+            undef, '0', undef, undef, undef, undef
+          ],
+          [ 'RAW', -3, 255, '\'', '\'', 'max length', 1, '0', 3,
+            undef, '0', undef, undef, undef, undef
+          ],
+          [ 'LONG', -1, '2147483647', '\'', '\'', undef, 1, 1, '0',
+            undef, '0', undef, undef, undef, undef
+          ],
+          [ 'CHAR', 1, 255, '\'', '\'', 'max length', 1, 1, 3,
+            undef, '0', '0', undef, undef, undef
+          ],
+          [ 'NUMBER', 3, 38, undef, undef, 'precision,scale', 1, '0', 3,
+            '0', '0', '0', undef, '0', 38
+          ],
+          [ 'DOUBLE', 8, 15, undef, undef, undef, 1, '0', 3,
+            '0', '0', '0', undef, undef, undef
+          ],
+          [ 'DATE', 11, 19, '\'', '\'', undef, 1, '0', 3,
+            undef, '0', '0', undef, '0', '0'
+          ],
+          [ 'VARCHAR2', 12, 2000, '\'', '\'', 'max length', 1, 1, 3,
+            undef, '0', '0', undef, undef, undef
+          ]
+        ];
+	return $ti;
+    }
+
     sub plsql_errstr {
 	# original version thanks to Bob Menteer
 	my $sth = shift->prepare_cached(q{
@@ -317,14 +384,18 @@ DBD::Oracle - Oracle database driver for the DBI module
 
   use DBI;
 
-  $dbh = DBI->connect("dbi:Oracle:", $user, $passwd);
+  $dbh = DBI->connect("dbi:Oracle:$dbname", $user, $passwd);
 
   # See the DBI module documentation for full details
 
+  # for some advanced uses you may need Oracle type values:
+  use DBD::Oracle qw(:ora_types);
+
+
 =head1 DESCRIPTION
 
 DBD::Oracle is a Perl module which works with the DBI module to provide
-access to Oracle databases.
+access to Oracle databases (both version 7 and 8).
 
 =head1 CONNECTING TO ORACLE
 
@@ -373,16 +444,18 @@ what is going on. (It's unfortunate that TWO_TASK couldn't be renamed,
 since it makes no sense to the end user, and doesn't have the ORACLE
 prefix).
 
-=head2 Using DBD::Oracle
+=head2 Connection Examples Using DBD::Oracle
 
 Below are various ways of connecting to an oracle database using
 SQL*Net 1.x and SQL*Net 2.x.  "Machine" is the computer the database is
 running on, "SID" is the SID of the database, "DB" is the SQL*Net 2.x
 connection descriptor for the database.
 
-  BEGIN { 
+B<Note:> Some of these formats don't work with Oracle 8.
+
+  BEGIN {
      $ENV{ORACLE_HOME} = '/home/oracle/product/7.x.x';
-     $ENV{TWO_TASK}    = 'DB'; 
+     $ENV{TWO_TASK}    = 'DB';
   }
   $dbh = DBI->connect('dbi:Oracle:','scott', 'tiger');
   #  - or -
@@ -429,12 +502,12 @@ a small proportion of people experience these problems.)
 [By Lane Sharman <lane@bienlogic.com>] I spent a LOT of time optimizing
 listener.ora and I am including it here for anyone to benefit from. My
 connections over tnslistener on the same humble Netra 1 take an average
-of 10-20 millies according to tnsping. If anyone knows how to make it
-better, please let me know!
+of 10-20 milli seconds according to tnsping. If anyone knows how to
+make it better, please let me know!
 
  LISTENER =
   (ADDRESS_LIST =
-        (ADDRESS = 
+        (ADDRESS =
           (PROTOCOL = TCP)
           (Host = aa.bbb.cc.d)
           (Port = 1521)
@@ -473,13 +546,13 @@ space for the system as well.
 
 5) Use large tuning settings and get lots of RAM. Check out all the
 parameters you can set in v$parameters because there are quite a few not
-documented you may to set in your initxxx.ora file. 
+documented you may to set in your initxxx.ora file.
 
 6) Use svrmgrl to control oracle from the command line. Write lots of small
 SQL scripts to get at V$ info.
 
   use DBI;
-  # Environmental variables used by Oracle 
+  # Environmental variables used by Oracle
   $ENV{ORACLE_SID}   = "xxx";
   $ENV{ORACLE_HOME}  = "/opt/oracle7";
   $ENV{EPC_DISABLED} = "TRUE";
@@ -506,7 +579,8 @@ If it generates any errors which look relevant then please talk to yor
 Oracle technical support (and not the dbi-users mailing list). Thanks.
 Thanks to Mark Dedlow for this information.
 
-=head2 International NLS / 8-bit text issues
+
+=head1 International NLS / 8-bit text issues
 
 If 8-bit text is returned as '?' characters or can't be inserted
 make sure the following environment vaiables are set correctly:
@@ -540,21 +614,21 @@ These PL/SQL examples come from: Eric Bartley <bartley@cc.purdue.edu>.
     CREATE OR REPLACE PACKAGE plsql_example
     IS
       PROCEDURE proc_np;
-   
+
       PROCEDURE proc_in (
           err_code IN NUMBER
       );
-   
+
       PROCEDURE proc_in_inout (
           test_num IN NUMBER,
           is_odd IN OUT NUMBER
       );
-   
+
       FUNCTION func_np
         RETURN VARCHAR2;
-   
+
     END plsql_example;
-  
+
     CREATE OR REPLACE PACKAGE BODY plsql_example
     IS
       PROCEDURE proc_np
@@ -563,7 +637,7 @@ These PL/SQL examples come from: Eric Bartley <bartley@cc.purdue.edu>.
       BEGIN
         SELECT USER INTO whoami FROM DUAL;
       END;
-   
+
       PROCEDURE proc_in (
         err_code IN NUMBER
       )
@@ -571,7 +645,7 @@ These PL/SQL examples come from: Eric Bartley <bartley@cc.purdue.edu>.
       BEGIN
         RAISE_APPLICATION_ERROR(err_code, 'This is a test.');
       END;
-   
+
       PROCEDURE proc_in_inout (
         test_num IN NUMBER,
         is_odd IN OUT NUMBER
@@ -580,7 +654,7 @@ These PL/SQL examples come from: Eric Bartley <bartley@cc.purdue.edu>.
       BEGIN
         is_odd := MOD(test_num, 2);
       END;
-   
+
       FUNCTION func_np
         RETURN VARCHAR2
       IS
@@ -589,38 +663,38 @@ These PL/SQL examples come from: Eric Bartley <bartley@cc.purdue.edu>.
         SELECT USER INTO ret_val FROM DUAL;
         RETURN ret_val;
       END;
-   
+
     END plsql_example;
   };
-  
+
   use DBI;
 
   my($db, $csr, $ret_val);
-  
+
   $db = DBI->connect('dbi:Oracle:database','user','password')
         or die "Unable to connect: $DBI::errstr";
-  
+
   # So we don't have to check every DBI call we set RaiseError.
   # See the DBI docs now if you're not familiar with RaiseError.
   $db->{RaiseError} = 1;
-  
+
   # Example 1
   #
   # Calling a PLSQL procedure that takes no parameters. This shows you the
   # basic's of what you need to execute a PLSQL procedure. Just wrap your
   # procedure call in a BEGIN END; block just like you'd do in SQL*Plus.
-  # 
+  #
   # p.s. If you've used SQL*Plus's exec command all it does is wrap the
   #      command in a BEGIN END; block for you.
-  
+
   $csr = $db->prepare(q{
     BEGIN
       PLSQL_EXAMPLE.PROC_NP;
     END;
   });
   $csr->execute;
-  
-  
+
+
   # Example 2
   #
   # Now we call a procedure that has 1 IN parameter. Here we use bind_param
@@ -632,13 +706,13 @@ These PL/SQL examples come from: Eric Bartley <bartley@cc.purdue.edu>.
   # placeholders (but few DBI drivers support them so they're not portable).
 
   my $err_code = -20001;
-  
+
   $csr = $db->prepare(q{
   	BEGIN
   	    PLSQL_EXAMPLE.PROC_IN(:err_code);
   	END;
   });
-  
+
   $csr->bind_param(":err_code", $err_code);
 
   # PROC_IN will RAISE_APPLICATION_ERROR which will cause the execute to 'fail'.
@@ -647,8 +721,8 @@ These PL/SQL examples come from: Eric Bartley <bartley@cc.purdue.edu>.
     $csr->execute;
   };
   print 'After proc_in: $@=',"'$@', errstr=$DBI::errstr, ret_val=$ret_val\n";
-  
-  
+
+
   # Example 3
   #
   # Building on the last example, I've added 1 IN OUT parameter. We still
@@ -662,13 +736,13 @@ These PL/SQL examples come from: Eric Bartley <bartley@cc.purdue.edu>.
 
   my $test_num = 5;
   my $is_odd;
-  
+
   $csr = $db->prepare(q{
   	BEGIN
   	    PLSQL_EXAMPLE.PROC_IN_INOUT(:test_num, :is_odd);
   	END;
   });
-  
+
   # The value of $test_num is _copied_ here
   $csr->bind_param(":test_num", $test_num);
 
@@ -676,9 +750,9 @@ These PL/SQL examples come from: Eric Bartley <bartley@cc.purdue.edu>.
 
   # The execute will automagically update the value of $is_odd
   $csr->execute;
-  
+
   print "$test_num is ", ($is_odd) ? "odd - ok" : "even - error!", "\n";
-  
+
 
   # Example 4
   #
@@ -688,44 +762,111 @@ These PL/SQL examples come from: Eric Bartley <bartley@cc.purdue.edu>.
   # we can access it's value after execute.
 
   my $whoami = "";
-  
+
   $csr = $db->prepare(q{
   	BEGIN
   	    :whoami := PLSQL_EXAMPLE.FUNC_NP;
   	END;
   });
-  
+
   $csr->bind_param_inout(":whoami", \$whoami, 20);
   $csr->execute;
   print "Your database user name is $whoami\n";
-  
+
   $db->disconnect;
 
 You can find more examples in the t/plsql.t file in the DBD::Oracle
 source directory.
 
 
+=head1 Oracle 8 Issues
+
+DBD::Oracle version 0.55 onwards can be built to use either the Oracle 7
+or Oracle 8 OCI (Oracle Call Interface) API functions. The new Oracle 8
+API is used by default and offers several advantages, including support
+for LOB type. Here's a quote from the Oracle OCI documentation:
+
+  The Oracle8 OCI has several enhancements to improve application
+  performance and scalability. Application performance has been improved
+  by reducing the number of client to server round trips required and
+  scalability improvements have been facilitated by reducing the amount
+  of state information that needs to be retained on the server side.
+
+(Note that the use of Oracle 8 OCI represents a major amount of
+rewriting of the internals of the drivers. There are bound to be some
+minor problems.)
+
+The DBD::Oracle module will avoid an explicit 'describe' operation
+prior to the execution of the statement unless the application requests
+information about the results (such as $sth->{NAME}). This reduces
+Communication with the server and increases performance.
+
+When fetching LOBs, they are treated just like LONGs and are subject to
+$sth->{LongReadLen} and $sth->{LongTruncOk}.
+
+When inserting or updating LOBs some *major* magic has to be performed
+behind the scenes to make it transparent.  Basically the driver has to
+refetch the newly inserted 'Lob Locators' before being able to write to
+them.  However, it works, and I've made it as fast as possible (just
+one extra server-round-trip per insert or update after the first).
+For the time being, only single-row LOB updates are supported.
+
+To insert or update a large LOB, DBD::Oracle has to know in advance
+that it is a LOB type. So you need to say:
+
+	$sth->bind_param($idx, $value, { ora_type => ORA_CLOB });
+
+The ORA_CLOB and ORA_BLOB constants can be imported using
+
+	use DBD::Oracle qw(:ora_types);
+
+or just use the corresponding integer values (112 and 113).
+
+To make scripts work with both Oracle7 and Oracle8, the Oracle7
+DBD::Oracle will treat the LOB ora_types as LONGs without error.
+So any code you may have now that looks like
+
+	$sth->bind_param($idx, $value, { ora_type => 8 });
+
+should change the 8 (LONG type) to ORA_CLOB or ORA_BLOB
+(or 112 or 113).
+
+
+=head1 Oracle on Linix
+
+To join the oracle-on-linux mailing list, see:
+
+  http://www.datamgmt.com/maillist.html
+  http://www.eGroups.com/list/oracle-on-linux
+  http://www.wmd.de/wmd/staff/pauck/misc/oracle_on_linux.html
+  mailto:oracle-on-linux-subscribe@egroups.com
+
+
 =head1 Commercial Oracle Tools
 
-Assorted tools and references for general information. No recommendation implied.
+Assorted tools and references for general information.
+No recommendation implied.
 
 PL/Vision from RevealNet and Steven Feuerstein.
 
 Platinum Technology: http://www.platinum.com/products/oracle.htm
 
+SoftTree Technologies: http://www.SoftTreeTech.com
+
 "Q" from Savant Corporation.
 
 http://www.databasegroup.com
 
 
+=head1 BUGS
+
+For $sth->{SCALE} and $sth->{PRECISION} Oracle always returns 0. Both
+with Oracle 7 and Oracle 8 OCI. Any idea why?
+
 =head1 SEE ALSO
 
 L<DBI>
 
-Linux uses should read:
-
-  http://www.wmd.de/wmd/staff/pauck/misc/oracle_on_linux.html
-
 =head1 AUTHOR
 
 DBD::Oracle by Tim Bunce. DBI by Tim Bunce.
@@ -740,6 +881,9 @@ without the prior approval of the author.
 
 =head1 ACKNOWLEDGEMENTS
 
+A great many people have helped me over the years. Far too many to
+name, but I thank them all.
+
 See also L<DBI/ACKNOWLEDGEMENTS>.
 
 =cut
